package form;

import javax.swing.*;
import javax.swing.border.StrokeBorder;

import Generics.PlaneGenericStorage;
import Generics.PlaneTrashCollection;
import Generics.PlanesGenericCollection;
import PlaneHard.FormPlaneConfig;
import drawing_objects.DrawingPlane;
import movement_strategy.DrawingObjectPlane;
import form.TxtSaveFilter;

import java.awt.*;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Objects;

public class FramePlaneCollection extends JFrame {
    private PlaneTrashCollection<DrawingPlane> trashCollection = new PlaneTrashCollection<>();
    private final PlaneGenericStorage storage;
    private JComponent pictureBoxCollection;
    private TextField textFieldNumber;
    private TextField textFieldStorageName;
    private JList<String> listStorages;
    private DefaultListModel<String> listModel;
    public FramePlaneCollection(){
        super("Набор самолетов");
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        createGui();
        pack();
        storage = new PlaneGenericStorage(pictureBoxCollection.getWidth(), pictureBoxCollection.getHeight());
        setVisible(true);
    }
    private void createGui(){
        //components initialisation
        pictureBoxCollection = new JComponent(){
            public void paintComponent(Graphics graphics){
                super.paintComponent(graphics);
                Graphics2D graphics2D = (Graphics2D) graphics;
                if (listStorages == null || storage == null)
                    return;
                var collection = storage.getCollection(listStorages.getSelectedValue());
                if (collection == null)
                    return;
                collection.ShowPlanes(graphics2D);
                super.repaint();
            }
        };
        JMenuBar menuFile = new JMenuBar();
        JMenu file = new JMenu("Файл");
        menuFile.add(file);
        JMenuItem saveFile = new JMenuItem("Сохранить");
        saveFile.addActionListener(e -> saveFile_Click());
        JMenuItem loadFile = new JMenuItem("Загрузить");
        loadFile.addActionListener(e -> loadFile_Click());
        JMenuItem saveCollection = new JMenuItem("Сохранить коллекцию");
        saveCollection.addActionListener(e -> saveCollection_Click());
        JMenuItem loadCollection = new JMenuItem("Загрузить коллекцию");
        loadCollection.addActionListener(e -> loadCollection_Click());
        file.add(saveCollection);
        file.add(loadCollection);
        file.add(saveFile);
        file.add(loadFile);
        super.setJMenuBar(menuFile);
        pictureBoxCollection.setPreferredSize(new Dimension(700, 600));
        JButton buttonAddPlane = new JButton("Добавить самолет");
        textFieldNumber = new TextField();
        JButton buttonRemovePlane = new JButton("Удалить самолет");
        JButton buttonRefreshCollection = new JButton("Обновить коллекцию");
        JButton buttonAddSet = new JButton("Добавить набор");
        JButton buttonDeleteSet = new JButton("Удалить набор");
        JButton buttonTrash = new JButton("Корзина");
        textFieldStorageName = new TextField();
        listModel = new DefaultListModel<>();
        JScrollPane scrollPane = new JScrollPane();
        listStorages= new JList<>(listModel);
        scrollPane.setViewportView(listStorages);
        //ActionListeners and ActionCommand addition
        buttonAddSet.addActionListener(e -> buttonAddSet_Click());
        buttonDeleteSet.addActionListener(e -> buttonDeleteSet_Click());
        buttonAddPlane.addActionListener(e -> buttonAddPlaneClick());
        buttonRemovePlane.addActionListener(e -> buttonRemovePlaneClick());
        buttonRefreshCollection.addActionListener(e -> buttonRefreshCollectionClick());
        buttonTrash.addActionListener(e->buttonTrashClick());
        //panels and constraints initialisation
        JPanel panelTools = new JPanel(new GridBagLayout());
        panelTools.setBorder(new StrokeBorder(new BasicStroke(3)));
        panelTools.setToolTipText("Инструменты");
        JPanel panelSets = new JPanel(new GridBagLayout());
        panelSets.setBorder(new StrokeBorder(new BasicStroke(3)));
        panelSets.setToolTipText("Наборы");
        GridBagConstraints constraints = new GridBagConstraints();
        constraints.insets.left = constraints.insets.right = 5;
        constraints.insets.top = constraints.insets.bottom = 5;
        constraints.fill = GridBagConstraints.BOTH;
        //addition to panelSets
        constraints.gridx = 0;
        constraints.gridy = 0;
        panelSets.add(textFieldStorageName, constraints);
        constraints.gridx = 0;
        constraints.gridy = 1;
        panelSets.add(buttonAddSet, constraints);
        constraints.gridx = 0;
        constraints.gridy = 2;
        panelSets.add(scrollPane, constraints);
        constraints.gridx = 0;
        constraints.gridy = 3;
        panelSets.add(buttonDeleteSet, constraints);
        //addition to panelTools
        constraints.gridx = 0;
        constraints.gridy = 0;
        panelTools.add(panelSets, constraints);
        constraints.gridx = 0;
        constraints.gridy = 1;
        panelTools.add(buttonAddPlane, constraints);
        constraints.gridx = 0;
        constraints.gridy = 2;
        panelTools.add(textFieldNumber, constraints);
        constraints.gridx = 0;
        constraints.gridy = 3;
        panelTools.add(buttonRemovePlane, constraints);
        constraints.gridx = 0;
        constraints.gridy = 4;
        panelTools.add(buttonRefreshCollection, constraints);
        constraints.gridx = 0;
        constraints.gridy = 5;
        panelTools.add(buttonTrash, constraints);
        //addition to frame
        setLayout(new BorderLayout());
        add(panelTools, BorderLayout.EAST);
        add(pictureBoxCollection, BorderLayout.CENTER);
    }

    private void saveFile_Click(){
        JFileChooser fc = new JFileChooser("C:\\Users\\123\\Desktop");
        fc.addChoosableFileFilter(new TxtSaveFilter());
        fc.addChoosableFileFilter(new TxtSaveFilter());
        int retrieval = fc.showSaveDialog(null);

        if (retrieval == JFileChooser.APPROVE_OPTION) {
            File file = new File(fc.getSelectedFile() + "." + "txt");

            try {
                storage.SaveData(file);
            } catch (IOException ex) {
                throw new RuntimeException(ex);
            }
        }
    }
    private void loadFile_Click() {
        JFileChooser fc = new JFileChooser("C:\\Users\\123\\Desktop");
        fc.addChoosableFileFilter(new TxtSaveFilter());
        int ret = fc.showDialog(null, "Открыть файл");
        if(ret == JFileChooser.APPROVE_OPTION){
            File file = fc.getSelectedFile();
            try {
                storage.LoadData(file);
                reloadObjects();
                super.repaint();
            } catch (IOException ex) {
                throw new RuntimeException(ex);
            }
        }
    }

    private void saveCollection_Click() {
        JFileChooser fc = new JFileChooser("C:\\Users\\123\\Desktop");
        fc.addChoosableFileFilter(new TxtSaveFilter());
        int retrieval = fc.showSaveDialog(null);

        if (retrieval == JFileChooser.APPROVE_OPTION) {
            File file = new File(fc.getSelectedFile() + "." + "txt");

            try {
                if(listStorages.getSelectedIndex() == -1) {
                    return;
                }
                storage.planeStorages.get(listStorages.getSelectedValue()).SaveData(file, listStorages.getSelectedValue());
                reloadObjects();
                super.repaint();
            } catch (IOException ex) {
                throw new RuntimeException(ex);
            }
        }
    }

    private void loadCollection_Click() {
        JFileChooser fc = new JFileChooser("C:\\Users\\123\\Desktop");
        fc.addChoosableFileFilter(new TxtSaveFilter());
        int ret = fc.showDialog(null, "Открыть файл");
        if(ret == JFileChooser.APPROVE_OPTION){
            File file = fc.getSelectedFile();
            try {
                storage.LoadCollection(file);
                reloadObjects();
                super.repaint();
            } catch (IOException ex) {
                throw new RuntimeException(ex);
            }
        }
    }

    private void reloadObjects(){
        int index = listStorages.getSelectedIndex();
        listModel.clear();
        ArrayList<String> keys = storage.Keys();
        for (String key : keys) {
            listModel.addElement(key);
        }
        if(listModel.size() > 0 && (index == -1 || index >= listModel.size()))
            listStorages.setSelectedIndex(0);
        else if(listModel.size() > 0)
            listStorages.setSelectedIndex(index);
    }
    public void buttonAddSet_Click() {
        if(textFieldStorageName.getText() == null ) {
            JOptionPane.showMessageDialog(this, "Не все данные заполнены");
            return;
        }
        String name = textFieldStorageName.getText();
        if (Objects.equals(name, "")) {
            JOptionPane.showMessageDialog(this, "Не все данные заполнены");
            return;
        }
        storage.AddSet(name);
        reloadObjects();
    }
    public void buttonDeleteSet_Click() {
        if (listStorages.getSelectedIndex() == -1)
            return;
        storage.DelSet(listStorages.getSelectedValue(), trashCollection);
        reloadObjects();
    }
    public void buttonAddPlaneClick() {
        if(listStorages.getSelectedIndex() == -1) {
            return;
        }
        PlanesGenericCollection<DrawingPlane, DrawingObjectPlane> drawingPlanes = storage.getCollection(listStorages.getSelectedValue());
        FormPlaneConfig framePlaneConfig = new FormPlaneConfig();
        framePlaneConfig.addButton.addActionListener(e -> {
            if (drawingPlanes.Insert(framePlaneConfig.drawingPlane)) {
                framePlaneConfig.dispose();
                framePlaneConfig.drawingPlane.pictureWidth = pictureBoxCollection.getWidth();
                framePlaneConfig.drawingPlane.pictureHeight = pictureBoxCollection.getHeight();
                JOptionPane.showMessageDialog(null, "Объект добавлен", "Информация", JOptionPane.INFORMATION_MESSAGE);
                pictureBoxCollection.repaint();
            }
            else {
                JOptionPane.showMessageDialog(null, "Не удалось добавить объект", "Информация", JOptionPane.INFORMATION_MESSAGE);
            }
        });
        framePlaneConfig.cancelButton.addActionListener(e -> framePlaneConfig.dispose());
    }
    private void buttonRemovePlaneClick(){
        if (listStorages.getSelectedIndex() == -1)
            return;
        var obj = storage.getCollection(listStorages.getSelectedValue());
        if (obj == null)
            return;
        int pos = Integer.parseInt(textFieldNumber.getText());
        var plane = obj.Get(pos);
        if (obj.Remove(pos)){
            JOptionPane.showMessageDialog(this, "Объект удален");
            trashCollection.Push(plane);
            pictureBoxCollection.repaint();
        }
        else{
            JOptionPane.showMessageDialog(this, "Не удалось удалить объект");
        }
    }
    private void buttonRefreshCollectionClick(){
        pictureBoxCollection.repaint();
    }
    private void buttonTrashClick(){
        if(trashCollection.GetSize() == 0)
            return;
        try {
            FrameAirBomber form = new FrameAirBomber();
            form.ChangePlane(trashCollection.Pop());
        } catch (Exception e){
            throw new RuntimeException();
        }
    }
}
